#!/bin/sh
#
#	Define a construct file with three Command lists.  In
#	addition to copying some files, the Command lists sleep
#	and echo lines to a common output file.  The resulting
#	common output file will have lines in different orders
#	when different -j values are used (or not).
#

: ${TMPDIR=/tmp}
work=${TMPDIR}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

: ${PROG=cons}
prog=${PROG}
progstr="$prog [-j]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X && test X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
: ${AR=ar}
: ${CC=cc}
: ${RANLIB=ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Command \$env '1', '1.sh', qq(
	rm -f %>
	sleep 1
	echo '1.sh message 1' >> output
	sleep 3
	echo '1.sh message 2' >> output
	cp %< %>
);
Command \$env '2', '2.sh', qq(
	rm -f %>
	sleep 2
	echo '2.sh message 1' >> output
	sleep 3
	echo '2.sh message 2' >> output
	cp %< %>
);
Command \$env '3', '3.sh', qq(
	rm -f %>
	sleep 3
	echo '3.sh message 1' >> output
	sleep 3
	echo '3.sh message 2' >> output
	cp %< %>
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >1.sh
#!/bin/sh
echo 'This is the 1.sh file.'
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >2.sh
#!/bin/sh
echo 'This is the 2.sh file.'
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >3.sh
#!/bin/sh
echo 'This is the 3.sh file.'
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog -j 3 . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - output <<'_EOF_'
1.sh message 1
2.sh message 1
3.sh message 1
1.sh message 2
2.sh message 2
3.sh message 2
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog -r . > cons.2.out 2>&1
if test $? -ne 0 ; then no_result ; fi
rm -f output
if test $? -ne 0 ; then no_result ; fi

$prog -j2 . >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
diff - output <<'_EOF_'
1.sh message 1
2.sh message 1
1.sh message 2
2.sh message 2
3.sh message 1
3.sh message 2
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

$prog -r . > cons.4.out 2>&1
if test $? -ne 0 ; then no_result ; fi
rm -f output
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
diff - output <<'_EOF_'
1.sh message 1
1.sh message 2
2.sh message 1
2.sh message 2
3.sh message 1
3.sh message 2
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

#
pass
