#!/bin/sh
#
#	Define a Construct file that (using Program) creates five
#	derived programs from C source.  The second and fourth C source
#	files contain syntax errors.  Execute Cons -j2; check that the
#	Cons exit code indicates an error, and that none of the second
#	through fifth derived programs were created (i.e., Cons stopped
#	at the error compiling the second program).  Execut Cons -j3;
#	check that the third program was created this time.  Replace the
#	second source file with one that works.  Execute Cons -j2 again.
#	Check that second program was built, but not the fourth and
#	fifth.  Replace the fourth source file with one that works.
#	Execute cons -j2 again.  Examine the build output to make sure
#	that only the fourth and fifth files were created.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

: ${TMPDIR=/tmp}
work=${TMPDIR}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

: ${PROG=cons}
prog=${PROG}
progstr="$prog [-j, rebuild after Program error]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X && test X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
: ${AR=ar}
: ${CC=cc}
: ${RANLIB=ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'aaa', 'aaa.c';
Program \$env 'bbb', 'bbb.c';
Program \$env 'ccc', 'ccc.c';
Program \$env 'ddd', 'ddd.c';
Program \$env 'eee', 'eee.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >aaa.c
main()
{
	printf("aaa.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >bbb.c
main()
{
	THIS LINE GENERATES A SYNTAX ERROR
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >ccc.c
main()
{
	printf("ccc.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >ddd.c
main()
{
	THIS LINE GENERATES A SYNTAX ERROR
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >eee.c
main()
{
	printf("eee.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog -j2 . >cons.1.out 2>&1
if test $? -eq 0 ; then fail 1 cons.1.out ; fi	# expect failure, -eq 0 
if test -f bbb ; then fail 1 cons.1.out ; fi
if test -f ccc ; then fail 1 cons.1.out ; fi
if test -f ddd ; then fail 1 cons.1.out ; fi
if test -f eee ; then fail 1 cons.1.out ; fi
./aaa >aaa.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - aaa.1.out <<'_EOF_'
aaa.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog -r . >cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi

$prog -j3 . >cons.3.out 2>&1
if test $? -eq 0 ; then fail 3 cons.3.out ; fi	# expect failure, -eq 0 
if test -f bbb ; then fail 3 cons.3.out ; fi
if test -f ddd ; then fail 3 cons.3.out ; fi
if test -f eee ; then fail 3 cons.3.out ; fi
./ccc >ccc.3.out 2>&1
if test $? -ne 0 ; then fail 3 ccc.3.out ; fi
diff - ccc.3.out <<'_EOF_'
ccc.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

cat <<'_EOF_' >bbb.c
main()
{
	printf("bbb.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog -j2 . >cons.4.out 2>&1
if test $? -eq 0 ; then fail 4 cons.4.out ; fi	# expect failure, -eq 0 
if test -f ddd ; then fail 4 cons.4.out ; fi
if test -f eee ; then fail 4 cons.4.out ; fi
./bbb >bbb.4.out 2>&1
if test $? -ne 0 ; then fail 4 bbb.4.out ; fi
diff - bbb.4.out <<'_EOF_'
bbb.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

cat <<'_EOF_' >ddd.c
main()
{
	printf("ddd.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog -j2 . >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
diff - cons.5.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $CC
$CC -c ddd.c -o ddd.o
$CC -o ddd ddd.o
$CC -c eee.c -o eee.o
$CC -o eee eee.o
_EOF_
if test $? -ne 0 ; then fail 5 ; fi
./ddd >ddd.5.out 2>&1
if test $? -ne 0 ; then fail 5 ddd.5.out ; fi
diff - ddd.5.out <<'_EOF_'
ddd.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi
./eee >eee.5.out 2>&1
if test $? -ne 0 ; then fail 5 eee.5.out ; fi
diff - eee.5.out <<'_EOF_'
eee.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

#
pass
