#!/bin/sh
#
#	Define a Construct file that (using Program) creates five
#	derived programs from C source.  The second and fourth C source
#	files contain syntax errors.  Execute Cons -j2; check that the
#	Cons exit code indicates an error, and that none of the second
#	through fifth derived programs were created (i.e., Cons stopped
#	at the error compiling the second program).  Execut Cons -j3;
#	check that the third program was created this time.  Replace the
#	second source file with one that works.  Execute Cons -j2 again.
#	Check that second program was built, but not the fourth and
#	fifth.  Replace the fourth source file with one that works.
#	Execute cons -j2 again.  Examine the build output to make sure
#	that only the fourth and fifth files were created.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

: ${TMPDIR=/tmp}
work=${TMPDIR}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

: ${PROG=cons}
prog=${PROG}
progstr="$prog [#include in generated .c file]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X && test X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
: ${AR=ar}
: ${CC=cc}
: ${RANLIB=ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Command \$env 'foo.c', 'foo.c.sh', qq(
	sh %< > %>
);
Command \$env 'foo.h', 'foo.h.sh', qq(
	sh %< > %>
);
Program \$env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo.c.sh
cat <<_end_of_foo_c_
#include	"foo.h"

main()
{
	printf("foo.c\n");
	printf("%s\n", FOO_H_STRING);
	exit (0);
}
_end_of_foo_c_
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo.h.sh
cat <<_end_of_foo_h_
#define	FOO_H_STRING	"foo.h"
_end_of_foo_h_
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
./foo >foo.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - foo.1.out <<'_EOF_'
foo.c
foo.h
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog -r . >cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi

$prog foo >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
./foo >foo.3.out 2>&1
if test $? -ne 0 ; then fail 3 ccc.3.out ; fi
diff - foo.3.out <<'_EOF_'
foo.c
foo.h
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

#
pass
